/*
 * ActivityDiagramGenerator.java
 *
 * Created on 22 de abril de 2006, 12:03
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.activitymodel;

import objectgen.usecase.IStep;
import objectgen.usecase.UseCaseXML;
import objectgen.usecase.UseCaseIteratorFactory;
import objectgen.usecase.StepBuilder;
import objectgen.usecase.pattern.*;

import java.util.Iterator;
import java.util.LinkedList;
import java.io.File;

import org.apache.log4j.*;

/**
* This class generates an Activity diagram from a use case in a XML file
*/
public class ActivityDiagramGenerator {

    /** Stores IControlFlow objects */
    private LinkedList lastNodeAdded;
    private ActivityDiagram ad;
    
    
    /** Creates a new instance of ActivityDiagramGenerator */
    public ActivityDiagramGenerator () {
        this.lastNodeAdded = new LinkedList();
        this.ad = new ActivityDiagram();
    }
    
    
    /** Generate an activity diagram from a use case */
    public ActivityDiagram generate (UseCaseXML uc) {
        Iterator ms;
        IStep mainS;
        Activity a;
        
        ms = uc.iteratorMainSequence ();
        while (ms.hasNext ()) {
            mainS = (IStep) ms.next ();
            this.generatePreCond(uc, mainS);
            this.createActivity (mainS);
            this.generatePostCond(uc, mainS);
            //addLastNodeLink(a);
            //this.lastNodeAdded.addLast (a);
        }
        
        addFinalActivities();
        //this.ad.finalActivity (/*this.lastNodeAdded*/);
        return this.ad;
    }

    
    //--- Helpper methods ------------------------------------------------
    
    /** Helpper methods.
     * Builds an activity class from a step 
     * of the main sequence **/
    private Activity createActivity(IStep step) {
        //log.fatal ("Method createActivity() not implemented yet.");
        
        log.debug ("Generating activity: " + step);
        Activity a = new Activity();
        a.setId (step.getId ());
        a.setDescription (step.getDescription () );
        a.setParticipant (step.getParticipant ());
        this.ad.addActivity (a);
        
        
        addLastNodeLink(a);
        this.lastNodeAdded.addLast (a);
        return a;
    }

    
    /** Helpper method
     * Traverse the precondition steps and calls cretaePreCond to build
     * a decision node for each precondition step.
     **/
    private void generatePreCond(UseCaseXML uc, IStep s) {
        IStep pre;
        // Obtenemos el iterador adecuado segn el tipo de plantillA
        //Iterator it = uc.iteratorPreSequence (s);
        Iterator it;
        it = UseCaseIteratorFactory.GetIteratorPreevaluatedSteps(uc, s);
        
        while(it.hasNext ()) {
            pre = (IStep) it.next ();
            createPreCond(pre);
        }
    }
    
    /** Helpper method
     **/
    private void generatePostCond(UseCaseXML uc, IStep s) {
        IStep pos;
        // Obtenemos el iterador adecuado segn el tipo de plantilla
        //Iterator it = uc.iteratorInvSequence (s);
        Iterator it;
        it = UseCaseIteratorFactory.GetIteratorPostevaluatedSteps(uc, s);
        
        while(it.hasNext ()) {
            pos = (IStep) it.next ();
            createPreCond(pos);
        }
    }

    
    /** Helpper method
     * Enter an empty activity or desicion and return the activity filled
     * with the information in the step.
     *
     * @param Activity a is an output parameter.
     */
    private void buildActivity(Activity a, IStep step, String stepText) {

        String template;
        
        a.setId (step.getId ());
        a.setDescription (stepText );

        // Si usamos la plantilla 2, todos los errores pertenecen al sistema.
        template = objectgen.configuration.Configuration.Get ("Template", "template01");
        
/*        if ( template.compareTo("template01") == 0 ) {
            a.setParticipant (step.getParticipant ());
            return;
        }*/
        
        if ((template.compareTo("template02") == 0) &&
            (step.getType () == IStep.INV) )
        {
            System.out.println ("Forcing system stereotype");
            a.setParticipant ("System");
            return;
        }
        
        
        if (step.getParticipant ().toLowerCase ().compareTo ("system") != 0)
        {
            String participant;
            StepBuilder sb = new StepBuilder();
            
            participant = sb.identifyParticipant (stepText);
            if (participant == null) {
                a.setParticipant (step.getParticipant ());
                return;
            }
            if (participant.toLowerCase ().compareTo ("system") != 0) {
                a.setParticipant (step.getParticipant ());
                return;
            }
            else
                a.setParticipant (participant);
            return;
        }

        // Estamos aqu cuando usamos la plantilla 1.
        //log.fatal ("You should not be here.");
        a.setParticipant (step.getParticipant ());
    }
    
    /** Helper method.
     * Builds the elements need to represent one alternative steps.
     * Elements are a decision node, transitions and
     * maybe aditional activities
     **/
    private void createPreCond(IStep pre) {
        
        Decision d;
        Alternative altTrue, altFalse;
        
        d = new Decision();
        altTrue = new Alternative();
        altTrue.setCondition ( pre.getCondition () );
        altFalse = new Alternative();
        altFalse.setCondition ( "not(" + pre.getCondition () +") " );
        
        // This is common code and should be refactorized.
        /*d.setId (pre.getId ());
        d.setDescription (pre.getDescription () );
        d.setParticipant (pre.getParticipant ());*/
        this.buildActivity (d, pre, pre.getDescription ());
        
        d.setAlternative (altTrue);
        d.setAlternative (altFalse);
            
        this.ad.addActivity (d);
        addLastNodeLink(d);
            
        // No conclusin
        if (pre.hasConclution () == false) {
            log.debug ("Generating: " + pre.getAction ());
            this.generateAlternatives (pre.getAction (),altTrue, altFalse, "0" );
        } // If no conclution
        else {
            // Aqu necesitamos saber el tipo de la accin y de la condicin 
            log.debug ("Generating: " + pre.getAction () + " / " + pre.getConclution ());
            
            // This is commond code and should be refactorized.
            Activity a = new Activity();
            /*a.setId (pre.getId ());
            a.setDescription (pre.getAction () );
            a.setParticipant (pre.getParticipant ());*/
            this.buildActivity (a, pre, pre.getAction ());
            
            altTrue.setNext (a.getId ());
            
            this.ad.addActivity (a);
            
            generateAlternatives (pre.getConclution (), a, altFalse, a.getId () );
            
        }
    }

    
    /** Classify and generate the true and
     *  false alternatives of a desicion 
     */
    private void generateAlternatives(String step, 
                                      IControlFlow altTrue, 
                                      IControlFlow altFalse,
                                      String stepId
        
        ) 
    {
        int type;
        AlternativeProcessor ap = new AlternativeProcessor();
        type = ap.findType (step);
                
        if (type == AlternativeProcessor.REPEAT ) {
            
            if ( ap.isAlternative (ap.getRepeatedId ()) ) {
                // The repeated step is an alternative
                altTrue.setNext( Decision.GenerateId (ap.getRepeatedId () ) );
            } else {
                // The repeated step is a main step
                altTrue.setNext(/*auxActivity*/ap.getRepeatedId ());
            }
            
            log.debug ("(REPEAT) Repeating step: " + ap.getRepeatedId () );
            // El false continua.
            this.lastNodeAdded.addLast (altFalse);
            return;
         }
                
         if (type == AlternativeProcessor.END) {
            log.debug ("(END) Generating end step.");
                altTrue.setNext (Activity.BuildFinal ());
             // Si el true termina, entonces el false continua.
             this.lastNodeAdded.addLast (altFalse);
             return;
         }
                
         if ( type == AlternativeProcessor.CONTINUE ) {
            //String step = this.ad.
            
            if (stepId.compareTo ("0") == 0)
                log.fatal ("Look at this");
            
            String id;
            int ip = stepId.indexOf ('.');
            id = stepId.substring (0, ip);
            log.debug ("(CONTINUE) ================> " +
                "Id actual step for continue: " 
                + id + " / " + stepId);
            
            int idIndex = Integer.getInteger (id).intValue ();
            idIndex++;
            
            String idNext = String.valueOf (idIndex);
            //this.
            // Debemos comporbar si este paso existe
            
            // Debemos comprobar si este paso tiene precondicin
            
           this.lastNodeAdded.addLast (altFalse);
           this.lastNodeAdded.addLast (altTrue);
           log.debug ("(CONTINUE) Adding last: " + altTrue + " / " + altFalse + " : " + this.lastNodeAdded.size ());
           return;
        }

        // No se si usaremos esto para algo
        if ( type == AlternativeProcessor.REPLACE ) {
            log.fatal ("(REPLACE) You should not be here.");   
            this.lastNodeAdded.addLast (altFalse);
           this.lastNodeAdded.addLast (altTrue);
           log.debug ("Adding last: " + altTrue + " / " + altFalse + " : " + this.lastNodeAdded.size ());
           return;
        }

        log.fatal ("You should not be here.");
    }

    
    /** Adds a link to the new activity **/
    private void addLastNodeLink(Activity a) {
            IControlFlow element;
            int i;
            int size = this.lastNodeAdded.size ();
            
            for (i = 0; i < size; i++) {
                element = (IControlFlow) this.lastNodeAdded.removeFirst ();
                log.debug ("Adding transition: " + element + " -----> " + a);
                this.ad.addTransition (element, a);
            }

    }

        /** Adds a link to the final node **/
        private void  addFinalActivities() {
            IControlFlow a;
            int i;
            
            for (i = 0; i < this.lastNodeAdded.size (); i++) {
                a = (IControlFlow) this.lastNodeAdded.removeFirst ();
                this.ad.addTransition (a, Activity.BuildFinal ());
            }
            //this.ad.finalActivity (/*this.lastNodeAdded*/);
        }


        
    //--- Log ------------------------------------------------------------
    static Logger log;
    static {
        log = Logger.getLogger (ActivityDiagramGenerator.class.getName());
    }

    
}


/** This class helps ActivityDiagramGenerator
 *  This class allows to identify the type of an alternative step
 *  and to extract information from it.
 */
class AlternativeProcessor {
   
    //--- Types of alternative steps -------------------------------------
    public static final int REPEAT = 1;
    public static final int END = 2;
    public static final int CONTINUE = 3;
    public static final int REPLACE = 4;
    
    //--- Attributes -----------------------------------------------------
    
    IPatternReader reader;
    IPatternMatcher match;
    String repeatedId;
    
    public AlternativeProcessor() { 
        this.match = new PatternMatcher();
        this.repeatedId = null;
    }
    
    /** 
     * @return the type of the steps, see const 
     **/
    public int findType(String s) {
        File f;
        String[] r;
        
        // 1. Search if the step is a repeat step
        f = objectgen.configuration.Configuration.GetFile("RepeatPatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null) {
            if (r.length > 0)
                this.repeatedId = r[0];
            else 
                ActivityDiagramGenerator.log.warn ("A repeat step without " + 
                    "repeatedId: " + r[0] /*+ "/" + r[1]*/);
            return AlternativeProcessor.REPEAT;
        }
        
        // 2. Search if the step is an end step
        f = objectgen.configuration.Configuration.GetFile("EndPatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null)
            return AlternativeProcessor.END;

        // 2. Search if the step is a continue step
        f = objectgen.configuration.Configuration.GetFile("ContinuePatternFile");
        this.reader = new PatternReader(f);
        r = this.match.match (s, this.reader);
        if ( r!= null)
            return AlternativeProcessor.CONTINUE;

        ActivityDiagramGenerator.log.fatal ("No type found for text: " + s);
        return -1;
    }

    /** 
     * If type is REPEAT, this method returns the id of the step
     **/
    public String getRepeatedId() {
        if (this.repeatedId == null)
            ActivityDiagramGenerator.log.fatal ("No repeatedId available. " + 
                                                "Returning null.");
        return this.repeatedId;
    }
    
    /**
     * If type is REPEAT, this method indicates if the step if from
     * the main sequence or from an alternative or erroneous sequence.
     */
    public boolean isAlternative(String id) {
        return id.indexOf ('.') != -1;
    }
}

