/*
 * JoinAlternativeFilter.java
 *
 * Created on 29 de mayo de 2006, 20:16
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.activitymodel;

import java.util.Iterator;
import org.apache.log4j.Logger;

/**
 * This class is a filter that find sequeces of alternatives in 
 *an activity diagram and join them.
 * 
 * This class is also a decorator of ActivityDiagram class
 *
 * @author Rincew
 */
public class JoinAlternativeFilter 
    extends ActivityDiagram
{
    
    ActivityDiagram ad;
    
    //--- Iterator attributes ------------------------------
    
    /** Creates a new instance of JoinAlternativeFilter */
    public JoinAlternativeFilter () {
    }
    
    /** Start point **/
    public void filter(ActivityDiagram ad) {
        this.ad = ad;
        
        while ( this.joinDesicion() )  ;
        
    }
    
    
    /** Returns true if the activity diagram was modified.
     * When this method returns false, all alternatives have been joined
     */
    boolean joinDesicion() {
        //boolean b;
        Iterator main;
        Iterator aux;
        Decision d_main;
        Decision d_aux;
        
        main = this.iteratorDecision ();
        while (main.hasNext ()) {
            d_main = (Decision) main.next ();
            aux = this.iteratorDecision ();
            while (aux.hasNext () ) {
                d_aux = (Decision) aux.next ();
                
                // Comprobamos si son el mismo
                if (d_main.getId ().compareTo (d_aux.getId ()) == 0)
                    continue;
                
                // Comprobamos si son consecutivas
                if ( this.consecutives (d_main, d_aux) ) {
                    this.joinDesicion(d_main, d_aux);
                    return true;
                }
            }
        }
        
        return false;
    }

    
    /** Returns true if the alternatives are consecutives 
     *  and has the same actor
     */
    boolean consecutives(Decision d1, Decision d2) {
        Iterator it;
        Alternative alt;
        boolean b = false;
        
        it = d1.iterator();
        while (it.hasNext ()) {
            alt = (Alternative) it.next ();
            if (alt.getNext ().compareTo (d2.getId ()) == 0) {
                b = true;
                break;
            }
                
        }
        
        if (b) {
            log.debug ("Two consecutive desicions found: \n" +
                d1+ "\n" + d2);
            b = (d1.getParticipant ().compareTo (d2.getParticipant ()) == 0);
            log.debug ("Join them: " + b);
        }
        return b;
    }
    
    
    /** Joind decisicion da in decision d
     */
    void joinDesicion(Decision d, Decision da) {
        Iterator it;
        Alternative alt;
        boolean verification = true;
        
        it = da.iterator ();
        while (it.hasNext ()) {
            alt = (Alternative) it.next ();
            if (this.isJoinable(alt)) {
                this.joinAlternative(d, alt);
                verification = false;
            } else {
                d.setAlternative (alt);
            }
        }
        
        this.changeNext (da, d);
        this.delete(da);
        
        if (verification) {
            log.warn ("No joinable alternative found in "+ da );
        }
    }
    
    
    /** Return true if the alternative has to be joined with 
     * other alternative and false if the alternatible is added
     * to the alternative.
     * Joinable alternatives start from not
     */
    boolean isJoinable(Alternative alt) {
        String condition;
        String token ="not";
        condition = alt.getCondition ().trim ().toLowerCase ();
        
        if (condition.startsWith (token))
            return true;
        
        return false;
    }
    
    
    /** **/
    void joinAlternative(Decision d, Alternative alt) {
        Iterator it;
        Alternative alt_aux;
        boolean verification = true;
        
        it = d.iterator ();
        while (it.hasNext () ) {
            alt_aux = (Alternative) it.next ();
            if ( this.isJoinable (alt_aux) ) {
                alt_aux.setCondition ( alt_aux.getCondition () +
                    " AND " + alt.getCondition () );
                alt_aux.setNext (alt.getNext ());
                verification = false;
                break;
            }
        }

        if (verification) {
            log.warn ("Nno joinable alternative found in " + d);
        }

        
    }
    
    
    /** Change next node from da to d
     */
    void changeNext (Decision da, Decision d) {
        Iterator it;
        
        Activity a;
        Decision d_aux;
        String next;
        
        it = this.ad.iteratorActivities ();
        while ( it.hasNext () ) {
            a = (Activity) it.next ();
            if (a == null)
                continue;
            
            if (a.isDesicion ()) {
             d_aux = (Decision) a;
             d_aux.changeAlternativeNext(da, d);
            } else {
                if (a.isNext (da) )
                    a.setNext (d);
            }
        }
        
    }
    
    
    /** Deletes a decision from the activity diagram
     */
    void delete(Decision da) {
        Object b;
        b = this.ad.da.remove ( da.getId () );
        
        if (b == null) {
            log.warn ("No decision found in activity diagram.");
        }
    }
    
    
    /** Testing purpouses only **/
    public String toString() {
        StringBuffer sb = new StringBuffer("");
        Iterator it;
        Decision d;
        
        sb.append ("JoinAlternativesFilter: \n");
        it = this.iteratorDecision ();
        while(it.hasNext ()) {
            d = (Decision) it.next ();
            sb.append (d + "\n");
        }
        
        return sb.toString ();
    }
    
    // --- Iterator ------------------

    public Iterator iteratorDecision() {
        return new decisionIterator();
    }

    //---
    class decisionIterator 
        implements Iterator
    {
        Object[] values;
        int valuesIndex;

        public decisionIterator() {
            this.values = ad.da.values ().toArray ();
            this.valuesIndex = 0;
        
        }
    
    
    public boolean hasNext () {
        Activity a = null;
        boolean exit = this.valuesIndex < this.values.length;
        
        while (exit) {
            a = (Activity) this.values[this.valuesIndex];
            if (a.isDesicion ())
                break;
            this.valuesIndex++;
            exit = this.valuesIndex < this.values.length;
            /*System.out.println ("Index:" + this.valuesIndex 
                + " / Size:" + this.values.length );*/
        }
        
        return  !(this.valuesIndex >= this.values.length);
    }

    public Object next () {
        Object o = this.values[this.valuesIndex];
        this.valuesIndex++;
        return o;
    }

    public void remove () {
        JoinAlternativeFilter.log.fatal ("Method not implemented.");
    }
    
    }
    
    //--- Log --------
    static Logger log;
    static {
        log = Logger.getLogger (JoinAlternativeFilter.class.getName());
    }

}
