package objectgen.activitymodel.xmi;

import java.io.*;
import java.util.Iterator;
import objectgen.activitymodel.*;

import org.apache.log4j.*;


public class XMIFileSaver {
  
  private FileWriter file;
  private int transitionId;
  private ActivityDiagram ad;

  //-------------------------------------------------------
  public XMIFileSaver() { 
    this.transitionId = 1;
    this.ad = null;
  }
  
  //-------------------------------------------------------

  /** This method saves a XMI representation of activity diagram
   * ad into file f.
   **/
  public void save(File f, ActivityDiagram ad) 
    throws IOException
  {
    IXMIHelpper helpper;
    this.file = new FileWriter(f);
    this.ad = ad;
    
    saveHeader();
    saveDiagramBegin();

    helpper = new XMIFileSaverActivityHelpper();
    helpper.saveElements (this.ad, this.file);

    /*saveActivitiesBegin();
    saveStartNode();
    //saveEndNode(); 
    saveNodes(ad); */
    saveActivitiesEnd();
    
    helpper = new XMIFileSaverTransitionHelpper();
    helpper.saveElements (this.ad, this.file);
    
   
    saveDiagramEnd() ;
    //saveStereotypes();
    
    helpper = new XMIFileSaverStereotypeHelpper();
    helpper.saveElements (this.ad, this.file);
    
    saveFileEnd();
    this.file.close();
  }
  
  /** 
   * ArgoUML XMI style
   **/
  private void saveHeader() 
    throws IOException
  {
    this.file.write("<?xml version = '1.0' encoding = 'UTF-8' ?> \n");
    this.file.write("<XMI xmi.version = '1.2' xmlns:UML = 'org.omg.xmi.namespace.UML' timestamp = 'Mon Mar 20 21:38:11 CET 2006'> \n");
    this.file.write("<XMI.header> \n");
    this.file.write("<XMI.documentation> \n");
    this.file.write("<XMI.exporter> ObjectGen - Test Objectives Generator</XMI.exporter> \n");
    this.file.write("<XMI.exporterVersion>0.0.2</XMI.exporterVersion> \n");
    this.file.write("</XMI.documentation> \n");
    this.file.write("<XMI.metamodel xmi.name=\"UML\" xmi.version=\"1.4\"/>  </XMI.header> \n");
  }
  
  /** **/
  private void saveDiagramBegin() 
    throws IOException
  {
    this.file.write("<XMI.content> \n");
    this.file.write("<UML:Model xmi.id = '.:0000000000000821' " +
        "name = 'Behaviour Model' " +
        "isSpecification = 'false' isRoot = 'false' " +
        "isLeaf = 'false' isAbstract = 'false'> \n");
    this.file.write("<UML:Namespace.ownedElement> \n");
    this.file.write("<UML:ActivityGraph xmi.id = '.:0000000000000822' " +
        "name='Behaviour Model' " + 
        "isSpecification = 'false' " +
        "context='.:0000000000000821'> \n");
    /*this.file.write("<UML:StateMachine.context> \n");
    this.file.write("<UML:Model xmi.idref = '.:0000000000000821'/> \n");
    this.file.write("</UML:StateMachine.context> \n");*/
  }


    
    
  /** **/
  private void saveDiagramEnd() 
    throws IOException
  {
    this.file.write("</UML:ActivityGraph> \n");
  }
  
    /**
     * *
     * @throws java.io.IOException 
     */
  private void saveActivitiesEnd() 
    throws IOException
  {
    this.file.write("  </UML:CompositeState.subvertex> \n");
    this.file.write(" </UML:CompositeState> \n");
    this.file.write("</UML:StateMachine.top> \n");
  }


 
  /*********** ***************/
  private void saveFileEnd() 
    throws IOException
  {
    //log.debug ("Saving file end.");
    this.file.write("</UML:Namespace.ownedElement> \n");
    this.file.write("</UML:Model> \n");
    this.file.write("</XMI.content> \n");
    this.file.write("</XMI> \n");
  }
 
  
    //--- Log ------------------------------------------------------------
    static Logger log;
    static {
        log = Logger.getLogger (ActivityDiagram.class.getName());
    }

}