/*
 * XMIFileSaverActivityHelpper.java
 *
 * Created on 20 de mayo de 2006, 13:02
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.activitymodel.xmi;

import java.io.*;
import java.util.Iterator;
import objectgen.activitymodel.*;

/**
 *
 * @author Rincew
 */
public class XMIFileSaverActivityHelpper 
    implements IXMIHelpper
{

    private ActivityDiagram ad;
    private FileWriter file;
    
    
    /** Creates a new instance of XMIFileSaverActivityHelpper */
    public XMIFileSaverActivityHelpper () {
        this.ad = null;
        this.file = null;
    }
 
    /** Start point **/
    public void saveElements(ActivityDiagram ad, FileWriter file) 
        throws IOException
    {
        this.ad = ad;
        this.file = file;
        
        saveActivitiesBegin();
        saveStartNode();
        saveNodes(ad);
    }
    
      /** **/
    private void saveActivitiesBegin() 
    throws IOException
 {
  
    this.file.write("<UML:StateMachine.top> \n");
    this.file.write("<UML:CompositeState xmi.id = '.:0000000000000823' " +
        "name = 'top' isSpecification = 'false' isConcurrent = 'false'> \n");
    this.file.write("<UML:CompositeState.subvertex> \n");
 }
  
  /** **/
  private void saveStartNode() 
    throws IOException
  {
      Activity firstActivity = this.ad.getFirstActivity ();
      Transition t;
      
      if (firstActivity == null) {
          XMIFileSaver.log.warn ("Activity diagram is empty.");
          return;
      }
      
      t = new Transition("UMLBeginNode", firstActivity.getId ());
      this.file.write("  <UML:Pseudostate xmi.id=\"UMLBeginNode\" " +
          "name='Use Case Start'" +
          " visibility=\"public\" isSpecification=\"false\" " +
          "container=\"UMLCompositeState.4\" kind=\"initial\"> \n");
      this.file.write("   <UML:Transition xmi.idref = '"+t.getId ()+"'/>\n" ); 
      this.file.write("  </UML:Pseudostate> \n");
  }

  
    
  /** **/
  private void saveEndNode() 
    throws IOException
  {
      //log.debug ("Saving end node.");
      this.file.write("<UML:FinalState xmi.id='End' " +
          "name='Use Case End' " +
          "visibility=\"public\" isSpecification=\"false\" container=\"UMLCompositeState.4\"/> \n");
  }
  
  /**  Traverse all nodes of the activity graph and calls to saveDesicion or
   * save Actvitity.
   **/
  private void saveNodes(ActivityDiagram nodes) 
    throws IOException
  {
    Iterator it;
    Activity a;
    Decision d;
    
    //log.debug ("Saving main sequence.");
    
    it = nodes.iteratorActivities();
    while( it.hasNext() ) {
        a = (Activity) it.next();
        if (a.isDesicion ()) {
            d = (Decision) a;
            saveDecision(d);
        } else
            saveActivity(a);
     }
  }
  
  /*****************************
   * Saves an activity and also end node.
   **/
  private void saveActivity(Activity n)
    throws IOException
  {
    // log.debug ("Saving node.");
      
    if ( n.getId().toLowerCase ().compareTo("end") == 0) {
        //log.debug ("Saving end node....");
        saveEndNode();
        saveIdTransitions(n);
        return;
    }

    this.file.write("<UML:ActionState xmi.id=\"" + n.getId() + "\" "); 
    this.file.write("name=\"" + n.getDescription() + "\" "); 
    this.file.write("visibility=\"public\" isSpecification=\"false\" " +
        "container=\"UMLCompositeState.4\" isDynamic=\"false\" ");
    this.file.write("> \n");
    
    // Prueba de estereotipos
    this.file.write(" <UML:ModelElement.stereotype> \n");
    this.file.write("  <UML:Stereotype xmi.idref = 'S.ActorStereotype'/> \n");
    this.file.write(" </UML:ModelElement.stereotype> \n");                    
    
    saveIdTransitions(n);
    this.file.write("</UML:ActionState>\n");
    
  }

  /*****************************
   * Saves a decision
   **/
  private void saveDecision(Decision n)
    throws IOException
  {
    // log.debug ("Saving node.");
    //                 <UML:Pseudostate xmi.id="UMLPseudostate.6" name="Decision1" visibility="public" isSpecification="false" container="UMLCompositeState.5" outgoing="UMLTransition.8" kind="branch"/>

    this.file.write("  <UML:Pseudostate xmi.id=\"" + n.getId() + "\" "); 
    this.file.write("name=\"" + n.getDescription() + "\" "); 
    this.file.write("visibility=\"public\" isSpecification=\"false\" container=\"UMLCompositeState.4\" isDynamic=\"false\" ");
    this.file.write(" kind=\"branch\" > \n");
    
    saveIdTransitions(n);
    
    this.file.write("  </UML:Pseudostate>\n"); 
  }
  
  /***********
   *
   **/
  void saveIdTransitions(Activity n) 
          throws IOException
  {
      Iterator it;
      Transition t;
      it = this.ad.iteratorTranstitionsBeginInActivity (n);
      
      this.file.write("   <UML:StateVertex.outgoing>\n");
      while(it.hasNext ()) {
          t = (Transition) it.next ();
          saveIdTransition(t);
      }
      this.file.write("   </UML:StateVertex.outgoing>\n");
      this.file.write("   <UML:StateVertex.incoming>\n");
      it = this.ad.iteratorTranstitionsEndInActivity (n);
      while(it.hasNext ()) {
          t = (Transition) it.next ();
          saveIdTransition(t);
      }
      this.file.write("   </UML:StateVertex.incoming>\n");
  }

 
  /****************************************
   */
  void saveIdTransition(Transition t) 
        throws IOException
  {
        this.file.write(" <UML:Transition xmi.idref = '" +
            t.getId () + "'/>\n" ); 
  }

  
}
