/*
 * AllScenariosCriterium.java
 *
 * Created on 13 de abril de 2006, 21:52
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package objectgen.testobjective;

import java.util.List;
import java.util.Iterator;
import objectgen.activitymodel.*;

import org.apache.log4j.Logger;

/**
 * This class implements the all scenarios criterium.
 * This means that all scenarios possible for none and one
 * repetition of every loop will be generated.
 * This class is also the based for other criteria.
 */
public class AllScenariosCriterium 
    implements ITestObjectiveCriterion {


    TestObjectiveList objectives;
    ActivityDiagram ad;
    
    /** Do nothing **/
    public AllScenariosCriterium () {    }
    
    /**** Start point
     * Generates a set of test objectives from an 
     * activity diagram. 
     **/
    public TestObjectiveList generate(ActivityDiagram ad) {
        this.ad = ad;
        this.objectives = new TestObjectiveList();
        
        AlternativeStore altStore = new AlternativeStore();
        TestObjective to = new TestObjective();
        this.traverse(ad.getFirstActivity (), to, altStore);
        
        log.debug ("altStore: "+ altStore);

        return this.objectives;
    }

   
    /** **/
    void traverse(Activity a, TestObjective to, AlternativeStore altStore) {
        Activity next;
        /*
        log.debug ("Traversing: "+a.getId () + " with path: "+to);
        */
        if (a.isFinal ()) {
            to.addActivity (a);
            this.objectives.add (to);
            this.checkCriterium();
            
            //this.ad.
            
            log.debug("Path found ("
                + this.objectives.size() + "): "
                + ">>"+to+"<<");
            return;
        }
        
        if (a.isDesicion () == false) {
            
            // Test if activity is repetaed.
            // If activity is repeated, then we have found a loop
            if (to.contains (a)) {
                // Take the last desicion / alternative and store it
                Decision d;
                d = to.getLastDecision ();
                altStore.addDecision (d, d.getFirstAlternative ());
                log.debug ("Loop found for activity: " + a 
                    +"\nStoring decision and alternative: " + d );
            }
            
            to.addActivity (a);
            next = this.ad.findActivity (a.getNext ());
            
            if (next == null) {
                log.warn ("Next null found for activity: " 
                    + a + " in path: " + to );
                return;
            }
            this.traverse ( next, to, altStore);
            return;
        }
        
        if (a.isDesicion ()) {
            this.traverseDesicion(a, to, altStore);
            return;
        }
    }

    /** **/
    void traverseDesicion(Activity a, TestObjective to, AlternativeStore altStore) {
        Decision d = (Decision) a.clone ();
        Decision d_aux;
        Alternative al;
        Alternative alt_aux;
        java.util.Iterator it;
        TestObjective to_aux;
        int numOfPaths = 0;
        /*
         this.altStore.addDecision (d);
         if (to.contains (d)) {
            log.info ("Loop found in " + d);
            traverseLoop(a, to);
            return ;
        }*/
        
        it = d.iterator ();
        while (it.hasNext ()) {
           
            numOfPaths++;
            //newPath = (Path)p.clone ();
            al = (Alternative) it.next ();
            
            // We must check if the alternative of this desicion is stored.
            // If it is stored, then is ignored
            if (altStore.stored (d, al)) {
                log.debug ("Loop decision found: " + al 
                    + "\n   Discarted path: " + numOfPaths);
                continue;
            }
            
            alt_aux = (Alternative) al.clone();
            d_aux = Decision.BuildDesicion (d, alt_aux);
            to_aux = (TestObjective) to.clone ();
            to_aux.addActivity (d_aux);
            
            log.debug ("traverseDesicion(...) - forking path " 
                + numOfPaths + " / " + d.alternativeNumber() + " "
                + to_aux + "\n      with alternative "+alt_aux );
            try {
     
                //AlternativeStore as = new AlternativeStore();
                this.traverse (this.ad.findActivity (alt_aux.getNext ()), to_aux, (AlternativeStore)altStore.clone());
            } catch (CloneNotSupportedException ex) {
                log.error ("Error when clonning");
                ex.printStackTrace();
            }
        }
    }

    /** 
     * This function verifies if the coverage criterium
     * has been obtained 
     * For all scenarios objective, there is not a 
     * checkCriterium
     **/
    void checkCriterium() {
        return ;
    }
    
    //--- Log --------
    static Logger log;
    static {
        log = Logger.getLogger (AllScenariosCriterium.class.getName());
    }
}
