/*
 */

package valuegen;

import objectgen.activitymodel.*;
import objectgen.activitymodel.xmi.XMIFileSaver;
import objectgen.testobjective.*;
import objectgen.testobjective.savers.*;
import objectgen.usecase.*;
import objectgen.configuration.Configuration;
import valuegen.data.*;
import valuegen.operations.*;


import java.io.*;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.BasicConfigurator;

/**
 */
public class VGCommandLine {
    
    /** **/
    public static void main(String[] args) 
    throws Exception
    {
        //--------------------------------------
        //---- ObjectGen code
        
        String objectivesFileName;
        File tof;
        String objectiveOutput;
        ITestObjectivesSaver tos;
        
        //--- Configure logs and program options -----------
        BasicConfigurator.configure ();
        Configuration.Create ();
        PropertyConfigurator.configure(Configuration.GetConfigPath () 
            + "logs.prop");
        
        
        System.out.println("VGCommandLine UseCase_File");
        System.out.println("--------------------------------------");
        
        if (args.length < 1) {
            System.err.println ("Not enough arguments.");
            return;
        }
        
        
        // 1. Activity diagram is builded
        File useCaseFile = new File(args[0]);
        System.out.println ("Use Case File: " + useCaseFile);
        
        UseCaseXML uc = new UseCaseXML();
        
        try {
            uc.load (useCaseFile);    
        } catch (UseCaseXMLException e) {
            System.out.println (e);
            System.exit (-1);
        }
        
        ActivityDiagramGenerator adg = new ActivityDiagramGenerator();
        ActivityDiagram ad = adg.generate (uc);

        if ( ("no".compareTo(Configuration.Get ("JoinAlternatives", "yes")) != 0) && 
             ("No".compareTo(Configuration.Get ("JoinAlternatives", "yes")) != 0) ) {
            JoinAlternativeFilter filter = new JoinAlternativeFilter();
            filter.filter (ad);
        }
        
        /*XMIFileSaver fs = new XMIFileSaver();
        fs.save (activityDiagramFile, ad);*/
        
        
        //--- Generating test objectives ------------------
        ITestObjectiveCriterion tog = CoverageCriteriaFactory.Get ();
        
        if (ad == null) {
            System.err.println ("No behaviour model generated.");
            System.exit (-1);
        }
        
        TestObjectiveList tol = tog.generate (ad);
        /*tof = new File(objectivesFileName);
        objectiveOutput = Configuration.Get ("ObjectiveOutput", "text");
        if (objectiveOutput.compareTo ("binary") == 0) {
            tos = new BinarySaver();
        } else {
            tos = new TextSaver();
        }
        tos.save (tof, tol);*/

        
        //------------------------------------------------
        //--- Valuegen code
        VariableList vl;
        VariableBuilder vb;
        VariableOutputXML vo;
        ConstraintBuilder cb;
        List cl;
        GenerateBSScript gbss;

        File f = new File("variables.xml");
        
        
        vb = new VariableBuilder();
        vl = (VariableList) vb.generate (ad);
        
        System.out.println ("Variables: " + vl.size ());
        vo = new VariableOutputXML();
        vo.generate (f, vl);
        
        cb = new ConstraintBuilder();
        cl = cb.generate (vl, tol);
        
        System.out.println ("Constrains: " + cl.size ());
        
        File output = new File("script.bsh");
        gbss = new GenerateBSScript();
        gbss.generate (cl, vl, output);
        
        
        System.out.println ("Ok.");

    }
    
    
}
