package valuegen.data;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * @version 1.0
 * @created 26-Dec-2006 13:58:52
 */
public class DependencyMatrix {

	private boolean[][] dependencyMatrix;
	/**
	 * Indicates the number of variables and the dimensions of the matrix
	 */
	private int variableNumber;
	/**
	 * For each name or ide of a variable, this attribute stores its index.
	 */
	private Map variableIndex;
        private int counter;
        
        

	public DependencyMatrix(int dimension){
            int i, j;
            
            this.variableNumber = dimension;
            dependencyMatrix = new boolean[dimension][dimension];
            for (i = 0; i < this.variableNumber; i++)
                for (j = 0; j < this.variableNumber; j++)
                    this.dependencyMatrix[i][j] = false;
            this.variableIndex = new HashMap();
            this.counter = 0;
	}


	/**
	 * 
	 * @param varOrg
	 * @param varDes
	 */
	public void setDependency(String varOrg, String varDes){
            Integer tmp;
            int i, j;
            
            tmp = (Integer)this.variableIndex.get (varOrg);
            if (tmp == null) {
                i = (this.counter++);
                this.variableIndex.put (varOrg, new Integer(i));
            } else {
                i = tmp.intValue ();
            }
            tmp = (Integer)this.variableIndex.get (varDes);
            if (tmp == null) {
                j = (this.counter++);
                this.variableIndex.put (varDes, new Integer(j));
            } else {
                j = tmp.intValue ();
            }
            
            log.debug ("Inserting (" + varOrg + ", " + varDes + 
                ") in ("+i+", "+j+")");
            this.dependencyMatrix[i][j] = true;
	}

	/**
	 * 
	 * @param v
	 */
	public VariableList getDependentVariablesFor(Variable v, VariableList vl){
            int index;
            Integer indexObj;
            VariableList vlr;
            int i, j;
            String varName;
            Variable vTmp;
            
            vlr = new VariableList();
            
            indexObj = (Integer) this.variableIndex.get (v.getName ());
            if (indexObj == null) {
                log.warn ("No index found for variable: " + v.getName ());
                return null;
            }
            index = indexObj.intValue ();
            
            for (j = 0; j < this.counter; j++) {
                if (j == index)  {
                    continue;
                }
                
                if (this.dependencyMatrix[index][j]) {
                    varName = this.getVariableNameForIndex(j);
                    vTmp = vl.getVariableWithName(varName);
                    vlr.add (vTmp);
                }
            }
            
            return vlr;
	}
        
        
        /** **/
        protected String getVariableNameForIndex(int index) {
            Map.Entry entry;
            Iterator itEntry;
            Integer iTmp;
            
            itEntry = this.variableIndex.entrySet ().iterator ();
            while (itEntry.hasNext ()) {
                entry = (Map.Entry) itEntry.next ();
                iTmp = (Integer) entry.getValue ();
                if (index == iTmp.intValue ())
                    return ((String) entry.getKey ());
            }
            
            log.fatal ("No variable name found for index: " + index);
            return "";
        }
        
        /** For testing purpouse
         */
        public String toString() {
            String s = "";
            int i, j;
            
            for (i = 0; i < this.variableNumber; i++ ) {
                for (j = 0; j < this.variableNumber; j++) 
                    s += " " + this.dependencyMatrix[i][j];
                s += "\n";
            }
            
            return s;
        }

        
    //--- Log ------------------------------------------------------------
    static Logger log;
    static {
        log = Logger.getLogger (DependencyMatrix.class.getName());
    }

}